@@ -37,12 +37,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_jasper_enc_debug);
 #define GST_CAT_DEFAULT gst_jasper_enc_debug
 
-static const GstElementDetails plugin_details =
-GST_ELEMENT_DETAILS ("Jasper JPEG2000 image encoder",
-    "Codec/Decoder/Image",
-    "Encodes video to JPEG2000 using jasper",
-    "Mark Nauwelaerts <mnauw@users.sf.net>");
-
 enum
 {
   ARG_0,
@@ -55,7 +49,7 @@ static GstStaticPadTemplate gst_jasper_enc_sink_template =
     GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_BGR "; "
         GST_VIDEO_CAPS_RGBx "; " GST_VIDEO_CAPS_xRGB "; "
         GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_xBGR "; "
-        GST_VIDEO_CAPS_YUV ("{ I420, YV12 }"))
+        GST_VIDEO_CAPS_YUV ("{ I420, YV12, v308 }"))
     );
 
 static GstStaticPadTemplate gst_jasper_enc_src_template =
@@ -71,16 +65,6 @@ static GstStaticPadTemplate gst_jasper_enc_src_template =
         "image/jp2")
     );
 
-static void gst_jasper_enc_base_init (gpointer g_class);
-static void gst_jasper_enc_class_init (GstJasperEncClass * klass);
-static void gst_jasper_enc_init (GstJasperEnc * filter,
-    GstJasperEncClass * klass);
-
-static void gst_jasper_enc_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_jasper_enc_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static void gst_jasper_enc_reset (GstJasperEnc * enc);
 static GstStateChangeReturn gst_jasper_enc_change_state (GstElement * element,
     GstStateChange transition);
@@ -115,29 +99,27 @@ gst_jasper_enc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jasper_enc_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jasper_enc_sink_template));
-  gst_element_class_set_details (element_class, &plugin_details);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jasper_enc_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jasper_enc_sink_template);
+  gst_element_class_set_details_simple (element_class,
+      "Jasper JPEG2000 image encoder", "Codec/Encoder/Image",
+      "Encodes video to JPEG2000 using jasper",
+      "Mark Nauwelaerts <mnauw@users.sf.net>");
 }
 
 /* initialize the plugin's class */
 static void
 gst_jasper_enc_class_init (GstJasperEncClass * klass)
 {
-  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   GST_DEBUG_CATEGORY_INIT (gst_jasper_enc_debug, "jp2kenc", 0,
       "Jasper JPEG2000 encoder");
 
-  gobject_class->set_property = gst_jasper_enc_set_property;
-  gobject_class->get_property = gst_jasper_enc_get_property;
-
   /* FIXME add some encoder properties */
 
   gstelement_class->change_state =
@@ -263,10 +245,10 @@ gst_jasper_enc_init_encoder (GstJasperEnc * enc)
   switch (enc->mode) {
     case GST_JP2ENC_MODE_J2C:
     case GST_JP2ENC_MODE_JPC:
-      enc->fmt = jas_image_strtofmt ("jpc");
+      enc->fmt = jas_image_strtofmt ((char *) "jpc");
       break;
     case GST_JP2ENC_MODE_JP2:
-      enc->fmt = jas_image_strtofmt ("jp2");
+      enc->fmt = jas_image_strtofmt ((char *) "jp2");
       break;
   }
 
@@ -307,8 +289,6 @@ static gboolean
 gst_jasper_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstJasperEnc *enc;
-  GstStructure *s;
-  const gchar *mimetype;
   GstVideoFormat format;
   gint width, height;
   gint fps_num, fps_den;
@@ -316,8 +296,6 @@ gst_jasper_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   gint i;
 
   enc = GST_JASPER_ENC (GST_PAD_PARENT (pad));
-  s = gst_caps_get_structure (caps, 0);
-  mimetype = gst_structure_get_name (s);
 
   /* get info from caps */
   if (!gst_video_format_parse_caps (caps, &format, &width, &height))
@@ -430,7 +408,7 @@ gst_jasper_enc_get_data (GstJasperEnc * enc, guint8 * data, GstBuffer ** outbuf)
 
   GST_LOG_OBJECT (enc, "all components written");
 
-  if (jas_image_encode (enc->image, stream, enc->fmt, ""))
+  if (jas_image_encode (enc->image, stream, enc->fmt, (char *) "sop"))
     goto fail_encode;
 
   GST_LOG_OBJECT (enc, "image encoded");
@@ -547,36 +525,6 @@ not_negotiated:
   }
 }
 
-static void
-gst_jasper_enc_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstJasperEnc *filter;
-
-  filter = GST_JASPER_ENC (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_jasper_enc_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstJasperEnc *filter;
-
-  filter = GST_JASPER_ENC (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstStateChangeReturn
 gst_jasper_enc_change_state (GstElement * element, GstStateChange transition)
 {
