@@ -332,8 +332,10 @@ gst_mpeg_video_parse (const guint8 * data, gsize size, guint offset)
     codoffsize->offset = gst_byte_reader_get_pos (&br) + offset;
 
     rsize = gst_byte_reader_get_remaining (&br);
-    if (rsize <= 0)
+    if (rsize <= 0) {
+      g_free (codoffsize);
       break;
+    }
 
     off = scan_for_start_codes (&br, 0, rsize);
 
