@@ -278,6 +278,8 @@ gst_csp_set_caps (GstBaseTransform * btrans, GstCaps * incaps,
 
     if (matrix && g_str_equal (matrix, "hdtv"))
       in_spec = COLOR_SPEC_YUV_BT709;
+    else if (matrix && g_str_equal (matrix, "jpeg"))
+      in_spec = COLOR_SPEC_YUV_JPEG;
     else
       in_spec = COLOR_SPEC_YUV_BT470_6;
   } else {
@@ -308,6 +310,8 @@ gst_csp_set_caps (GstBaseTransform * btrans, GstCaps * incaps,
 
     if (matrix && g_str_equal (matrix, "hdtv"))
       out_spec = COLOR_SPEC_YUV_BT709;
+    else if (matrix && g_str_equal (matrix, "jpeg"))
+      out_spec = COLOR_SPEC_YUV_JPEG;
     else
       out_spec = COLOR_SPEC_YUV_BT470_6;
   } else {
@@ -412,10 +416,10 @@ gst_csp_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_csp_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_csp_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_csp_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_csp_sink_template);
 
   gst_element_class_set_details_simple (element_class,
       " Colorspace converter", "Filter/Converter/Video",
