@@ -97,13 +97,12 @@ gst_mxf_mux_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
   const GstPadTemplate **p;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_templ));
+  gst_element_class_add_static_pad_template (element_class, &src_templ);
 
   p = mxf_essence_element_writer_get_pad_templates ();
   while (p && *p) {
     gst_element_class_add_pad_template (element_class,
-        (GstPadTemplate *) gst_object_ref (GST_OBJECT (*p)));
+        (GstPadTemplate *) (GST_OBJECT (*p)));
     p++;
   }
 
@@ -376,6 +375,18 @@ gst_mxf_mux_setcaps (GstPad * pad, GstCaps * caps)
   return ret;
 }
 
+static char *
+gst_mxf_mux_create_pad_name (GstPadTemplate * templ, guint id)
+{
+  GString *string;
+
+  string = g_string_new (GST_PAD_TEMPLATE_NAME_TEMPLATE (templ));
+  g_string_truncate (string, string->len - 2);
+  g_string_append_printf (string, "%u", id);
+
+  return g_string_free (string, FALSE);
+}
+
 static GstPad *
 gst_mxf_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * pad_name)
@@ -397,9 +408,12 @@ gst_mxf_mux_request_new_pad (GstElement * element,
     GST_ERROR_OBJECT (mux, "Not our template");
     return NULL;
   }
-
+#if GLIB_CHECK_VERSION(2,29,5)
+  pad_number = g_atomic_int_add ((gint *) & mux->n_pads, 1);
+#else
   pad_number = g_atomic_int_exchange_and_add ((gint *) & mux->n_pads, 1);
-  name = g_strdup_printf (GST_PAD_TEMPLATE_NAME_TEMPLATE (templ), pad_number);
+#endif
+  name = gst_mxf_mux_create_pad_name (templ, pad_number);
 
   GST_DEBUG_OBJECT (mux, "Creating pad '%s'", name);
   pad = gst_pad_new_from_template (templ, name);
