@@ -71,6 +71,8 @@ struct _GstH264Parse
   guint align;
   guint format;
   guint current_off;
+  gboolean packetized_last;
+  gboolean packetized_chunked;
 
   GstClockTime last_report;
   gboolean push_codec;
@@ -90,11 +92,12 @@ struct _GstH264Parse
   GstClockTime dts;
   /* dts at start of last buffering period */
   GstClockTime ts_trn_nb;
+  gboolean do_ts;
 
   /* frame parsing */
   /*guint last_nal_pos;*/
   /*guint next_sc_pos;*/
-  gint idr_pos;
+  gint idr_pos, sei_pos;
   gboolean update_caps;
   GstAdapter *frame_out;
   gboolean keyframe;
